Class: SyntaxTree::Haml::Format::PartList

Inherits:
Object
  • Object
show all
Defined in:
lib/syntax_tree/haml/format.rb

Overview

When formatting a tag, there are a lot of different kinds of things that can be printed out. There's the tag name, the attributes, the content, etc. This object is responsible for housing all of those parts.

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(node) ⇒ PartList

Returns a new instance of PartList.



155
156
157
158
# File 'lib/syntax_tree/haml/format.rb', line 155

def initialize(node)
  @node = node
  @parts = []
end

Instance Attribute Details

#node ⇒ Object (readonly)

Returns the value of attribute node.



153
154
155
# File 'lib/syntax_tree/haml/format.rb', line 153

def node
  @node
end

#parts ⇒ Object (readonly)

Returns the value of attribute parts.



153
154
155
# File 'lib/syntax_tree/haml/format.rb', line 153

def parts
  @parts
end

Instance Method Details

#<<(part) ⇒ Object



160
161
162
# File 'lib/syntax_tree/haml/format.rb', line 160

def <<(part)
  parts << part
end

#empty? ⇒ Boolean

Returns:

  • (Boolean)


164
165
166
# File 'lib/syntax_tree/haml/format.rb', line 164

def empty?
  parts.empty?
end

#format(q) ⇒ Object



168
169
170
171
172
173
174
175
176
177
178
179
180
# File 'lib/syntax_tree/haml/format.rb', line 168

def format(q)
  if empty? && node.value[:name] == "div"
    # If we don't have any other parts to print and the tag is a div
    # then we need to make sure to add that to the beginning. Otherwise
    # it's implied by the presence of other operators.
    q.text("%div")
  else
    parts.inject(0) do |align, part|
      part.format(q, align)
      align + part.length
    end
  end
end